/**************************************************************************
 *   MudClient - alias.c                                                  *
 *   Copyright (C) 1997  Christophe Roux                                  *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 1, or (at your option)  *
 *   any later version.                                                   *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, write to the Free Software          *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
 *                                                                        *
 **************************************************************************/

#include <string.h>
#include <time.h>
#include <stdio.h>
#include <unistd.h>
#include "alias.h"
#include "mudcl.h"

extern char conn;
extern char path[];

struct alias alias_tab[MAX_ALIAS];
int nb_alias = 0;

char strtmp[ACTION_SIZE],strtmp2[ACTION_SIZE];

char aliasfile[FILENAME];

int find_alias(char *name)
{
  int i;

  if (!nb_alias)
    return 0;
  for (i = 0 ; i < nb_alias ; i++)
    if (!strcmp(name,alias_tab[i].name))
      return 1;
  return 0;
}

void list_alias(char *param)
{
  int i;
  char *ptr;

  putscr("\n*** Aliases List :\n");
  for (i = 0 ; i < nb_alias ; i++)
  {
    sprintf(strtmp,"%s %s",alias_tab[i].name,alias_tab[i].action);
    ptr = strtmp;
    while (*ptr)
    {
      if ((*ptr == '%') && (*(ptr + 1) == 's'))
      {
	*ptr++ = '$';
	*ptr++ = '*';
      }
      else if ((*ptr == '\r') && (*(ptr + 1) == '\n'))
      {
	*ptr++ = ':';
	*ptr++ = ':';
      }
      else
	ptr++;
    }
    putscr("*** %s\n",strtmp);
  }
  putscr("*** End of aliases list\n");
}

void add_alias(char *param)
{
  char *cmd,*alias_param,*ptr;
  int nb_alias_param;

  nb_alias_param = 0;
  cmd = param;
  
  if (nb_alias == MAX_ALIAS)
  {
    putscr("\n*** Error : Aliases table is full\n");
    return;
  }    
  if ((*cmd) && (strlen(param) < ACTION_SIZE))
  {
    alias_param = param;
    while ((*alias_param != '\0') && (*alias_param != ' '))
      alias_param++;
    if (*alias_param)
    {
      *alias_param = '\0';
      alias_param++;
      while(*alias_param == ' ')
	alias_param++;
    }
    if (*alias_param)
    {
      ptr = alias_param;
      while (*alias_param)
      {
	if ((*alias_param == '$') && (*(alias_param + 1) == '*'))
	{
	  *alias_param++ = '%';
	  *alias_param++ = 's';
	  nb_alias_param++;
	}
	else if ((*alias_param == ':') && (*(alias_param + 1) == ':'))
	{
	  *alias_param++ = '\r';
	  *alias_param++ = '\n';
	}
	else
	  alias_param++;
      }
      if (find_alias(cmd))
      {
	putscr("\n*** Error : Alias %s already in use\n",cmd);
	return;
      }	
      if (strlen(cmd) >= NAME_SIZE)
      {
	putscr("\n*** Error : Alias name is too long\n");
	return;
      }	
      if (nb_alias_param > 10)
      {
	putscr("\n*** Error : Too much parameter\n");
	return;
      }	
      strcpy(alias_tab[nb_alias].name,cmd);
      strcpy(alias_tab[nb_alias].action,ptr);
      alias_tab[nb_alias].nb = nb_alias_param;
      nb_alias++;
      return;
    }
  }
  putscr("\n*** Error : Syntax error or action too long\n");
}

void load_alias(char *param)
{
  FILE *stream;

  if (*param != '/')
  {
    sprintf(aliasfile,"%s/%s",path,param);
    stream = fopen(aliasfile,"r");
  }
  else
    stream = fopen(param,"r");
  if (!stream)
  {
    putscr("\n*** Error : %s file not found\n",param);
    return;
  }
  while (!feof(stream)) {
    *strtmp = '\0';
    fgets(strtmp,ACTION_SIZE,stream);
    if (strtmp[strlen(strtmp) - 1] == '\n')
      strtmp[strlen(strtmp) - 1] = '\0';
    if (!feof(stream))
      if (*strtmp > '!')
	add_alias(strtmp);
  }
  fclose(stream);
}

void save_alias(char *param)
{
  int i;
  char *ptr;
  FILE *stream;

  if (*param != '/')
  {
    sprintf(aliasfile,"%s/%s",path,param);
    stream = fopen(aliasfile,"w");
  }
  else
    stream = fopen(param,"w");
  if (!stream)
  {
    putscr("\n*** Error : Can't write %s\n",param);
    return;
  }
  for (i = 0 ; i < nb_alias ; i++)
  {
    sprintf(strtmp,"%s %s",alias_tab[i].name,alias_tab[i].action);
    ptr = strtmp;
    while (*ptr)
    {
      if ((*ptr == '%') && (*(ptr + 1) == 's'))
      {
	*ptr++ = '$';
	*ptr++ = '*';
      }
      else if ((*ptr == '\r') && (*(ptr + 1) == '\n'))
      {
	*ptr++ = ':';
	*ptr++ = ':';
      }
      else
	ptr++;
    }
    fprintf(stream,"%s\n",strtmp);
  }
  fclose(stream);
}

void rm_alias(char *param)
{
  int i;

  if (!nb_alias)
  {
    putscr("\n*** Error : No alias define\n");
    return;
  }
  for (i = 0 ; i < nb_alias ; i++)
  {
    if (!strcmp(param,alias_tab[i].name))
    {
      alias_tab[i] = alias_tab[nb_alias - 1];
      nb_alias--;
      return;
    }
  }
  putscr("\n*** Error : This alias was not define\n");
}

int is_alias(char *line,int sock)
{
  int i;
  char *alias_param;

  *strtmp = '\0';
  sscanf(line,"%s",strtmp);
  for (i = 0 ; i < nb_alias ; i++)
  {
    if (!strcmp(strtmp,alias_tab[i].name))
    {
      alias_param = line;
      while ((*alias_param != '\0') && (*alias_param != ' '))
	alias_param++;
      if (*alias_param)
      {
	while(*alias_param == ' ')
	  alias_param++;
      }
      strcpy(strtmp,alias_tab[i].action);
      if (*alias_param)
      {
	switch(alias_tab[i].nb)
	{
	case 0:	
	  strcpy(strtmp2,strtmp);
	  break;
	case 1:
	  sprintf(strtmp2,strtmp,alias_param);
	  break;
	case 2:
	  sprintf(strtmp2,strtmp,alias_param,alias_param);
	  break;
	case 3:
	  sprintf(strtmp2,strtmp,alias_param,alias_param,alias_param);
	  break;
	case 4:
	  sprintf(strtmp2,strtmp,alias_param,alias_param,alias_param,alias_param);
	  break;
	case 5:
	  sprintf(strtmp2,strtmp,alias_param,alias_param,alias_param,alias_param,alias_param);
	  break;
	case 6:
	  sprintf(strtmp2,strtmp,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param);
	  break;
	case 7:
	  sprintf(strtmp2,strtmp,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param);
	  break;
	case 8:
	  sprintf(strtmp2,strtmp,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param);
	  break;
	case 9:
	  sprintf(strtmp2,strtmp,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param);
	  break;
	case 10:
	  sprintf(strtmp2,strtmp,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param,alias_param);
	  break;
	}
      }
      else if (!alias_tab[i].nb)
      {
	strcpy(strtmp2,strtmp);
      }
      else
      {
	putscr("\n*** Error : This alias need a parameter\n");
	return 1;
      }
      if (!is_alias(strtmp2,sock))
      {
	if ((int) write(sock,strtmp2,strlen(strtmp2)) < 0)
	  conn=0;
	send_crnl(sock);
      }
      return 1;
    }
  }
  return 0;
}
